@@ -1,4 +1,7 @@
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2010 litl, LLC
 
+const ByteArray = imports.byteArray;
 const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
 
@@ -6,24 +9,23 @@ let loop = GLib.MainLoop.new(null, false);
 
 function cat(filename) {
     let f = Gio.file_new_for_path(filename);
-    f.load_contents_async(null, function(f, res) {
+    f.load_contents_async(null, (obj, res) => {
         let contents;
         try {
-            contents = f.load_contents_finish(res)[1];
+            contents = obj.load_contents_finish(res)[1];
         } catch (e) {
             logError(e);
             loop.quit();
             return;
         }
-        print(contents);
+        print(ByteArray.toString(contents));
         loop.quit();
     });
 
     loop.run();
 }
 
-if (ARGV.length != 1) {
+if (ARGV.length !== 1)
     printerr('Usage: gio-cat.js filename');
-} else {
+else
     cat(ARGV[0]);
-}
