@@ -5,6 +5,7 @@
 #include <config.h>  // for PACKAGE_STRING
 
 #include <locale.h>  // for setlocale, LC_ALL
+#include <stdint.h>
 #include <stdlib.h>  // for exit
 #include <string.h>  // for strcmp, strlen
 
@@ -19,6 +20,7 @@
 #include <glib.h>
 
 #include <gjs/gjs.h>
+#include <gjs/jsapi-util.h>
 
 static char **include_path = NULL;
 static char **coverage_prefixes = NULL;
@@ -28,6 +30,7 @@ static char *command = NULL;
 static gboolean print_version = false;
 static gboolean print_js_version = false;
 static gboolean debugging = false;
+static gboolean exec_as_module = false;
 static bool enable_profiler = false;
 
 static gboolean parse_profile_arg(const char *, const char *, void *, GError **);
@@ -41,6 +44,7 @@ static GOptionEntry entries[] = {
     { "coverage-prefix", 'C', 0, G_OPTION_ARG_STRING_ARRAY, &coverage_prefixes, "Add the prefix PREFIX to the list of files to generate coverage info for", "PREFIX" },
     { "coverage-output", 0, 0, G_OPTION_ARG_STRING, &coverage_output_path, "Write coverage output to a directory DIR. This option is mandatory when using --coverage-prefix", "DIR", },
     { "include-path", 'I', 0, G_OPTION_ARG_STRING_ARRAY, &include_path, "Add the directory DIR to the list of directories to search for js files.", "DIR" },
+    { "module", 'm', 0, G_OPTION_ARG_NONE, &exec_as_module, "Execute the file as a module." },
     { "profile", 0, G_OPTION_FLAG_OPTIONAL_ARG | G_OPTION_FLAG_FILENAME,
         G_OPTION_ARG_CALLBACK, reinterpret_cast<void *>(&parse_profile_arg),
         "Enable the profiler and write output to FILE (default: gjs-$PID.syscap)",
@@ -95,7 +99,7 @@ static void
 check_script_args_for_stray_gjs_args(int           argc,
                                      char * const *argv)
 {
-    GError *error = NULL;
+    GjsAutoError error;
     char **new_coverage_prefixes = NULL;
     char *new_coverage_output_path = NULL;
     char **new_include_paths = NULL;
@@ -120,9 +124,8 @@ check_script_args_for_stray_gjs_args(int           argc,
     g_option_context_set_ignore_unknown_options(script_options, true);
     g_option_context_set_help_enabled(script_options, false);
     g_option_context_add_main_entries(script_options, script_check_entries, NULL);
-    if (!g_option_context_parse_strv(script_options, &argv_copy, &error)) {
+    if (!g_option_context_parse_strv(script_options, &argv_copy, error.out())) {
         g_warning("Scanning script arguments failed: %s", error->message);
-        g_error_free(error);
         g_strfreev(argv_copy);
         return;
     }
@@ -161,19 +164,31 @@ check_script_args_for_stray_gjs_args(int           argc,
 int define_argv_and_eval_script(GjsContext* js_context, int argc,
                                 char* const* argv, const char* script,
                                 size_t len, const char* filename) {
+    gjs_context_set_argv(js_context, argc, const_cast<const char**>(argv));
+
     GError* error = nullptr;
-
-    /* prepare command line arguments */
-    if (!gjs_context_define_string_array(
-            js_context, "ARGV", argc, const_cast<const char**>(argv), &error)) {
-        g_critical("Failed to define ARGV: %s", error->message);
-        g_clear_error(&error);
-        return 1;
-    }
-
     /* evaluate the script */
-    int code;
-    if (!gjs_context_eval(js_context, script, len, filename, &code, &error)) {
+    int code = 0;
+    if (exec_as_module) {
+        GjsAutoUnref<GFile> output = g_file_new_for_commandline_arg(filename);
+        GjsAutoChar uri = g_file_get_uri(output);
+        if (!gjs_context_register_module(js_context, uri, uri, &error)) {
+            g_critical("%s", error->message);
+            g_clear_error(&error);
+            code = 1;
+        }
+
+        uint8_t code_u8 = 0;
+        if (!code &&
+            !gjs_context_eval_module(js_context, uri, &code_u8, &error)) {
+            code = code_u8;
+
+            if (!g_error_matches(error, GJS_ERROR, GJS_ERROR_SYSTEM_EXIT))
+                g_critical("%s", error->message);
+            g_clear_error(&error);
+        }
+    } else if (!gjs_context_eval(js_context, script, len, filename, &code,
+                                 &error)) {
         if (!g_error_matches(error, GJS_ERROR, GJS_ERROR_SYSTEM_EXIT))
             g_critical("%s", error->message);
         g_clear_error(&error);
@@ -185,7 +200,7 @@ int
 main(int argc, char **argv)
 {
     GOptionContext *context;
-    GError *error = NULL;
+    GjsAutoError error;
     GjsContext *js_context;
     GjsCoverage *coverage = NULL;
     char *script;
@@ -207,7 +222,7 @@ main(int argc, char **argv)
     g_option_context_set_help_enabled(context, false);
 
     g_option_context_add_main_entries(context, entries, NULL);
-    if (!g_option_context_parse_strv(context, &argv_copy, &error))
+    if (!g_option_context_parse_strv(context, &argv_copy, error.out()))
         g_error("option parsing failed: %s", error->message);
 
     /* Split options so we pass unknown ones through to the JS script */
@@ -240,9 +255,10 @@ main(int argc, char **argv)
     print_version = false;
     print_js_version = false;
     debugging = false;
+    exec_as_module = false;
     g_option_context_set_ignore_unknown_options(context, false);
     g_option_context_set_help_enabled(context, true);
-    if (!g_option_context_parse_strv(context, &gjs_argv, &error)) {
+    if (!g_option_context_parse_strv(context, &gjs_argv, error.out())) {
         char* help_text = g_option_context_get_help(context, true, nullptr);
         g_printerr("%s\n\n%s\n", error->message, help_text);
         g_free(help_text);
@@ -262,6 +278,7 @@ main(int argc, char **argv)
         exit(0);
     }
 
+    GjsAutoChar program_path = nullptr;
     gjs_argc = g_strv_length(gjs_argv);
     if (command != NULL) {
         script = command;
@@ -269,6 +286,12 @@ main(int argc, char **argv)
         filename = "<command line>";
         program_name = gjs_argv[0];
     } else if (gjs_argc == 1) {
+        if (exec_as_module) {
+            g_warning(
+                "'-m' requires a file argument.\nExample: gjs -m main.js");
+            exit(1);
+        }
+
         script = g_strdup("const Console = imports.console; Console.interact();");
         len = strlen(script);
         filename = "<stdin>";
@@ -278,10 +301,13 @@ main(int argc, char **argv)
         /* All unprocessed options should be in script_argv */
         g_assert(gjs_argc == 2);
         error = NULL;
-        if (!g_file_get_contents(gjs_argv[1], &script, &len, &error)) {
+        GjsAutoUnref<GFile> input = g_file_new_for_commandline_arg(gjs_argv[1]);
+        if (!g_file_load_contents(input, nullptr, &script, &len, nullptr,
+                                  error.out())) {
             g_printerr("%s\n", error->message);
             exit(1);
         }
+        program_path = g_file_get_path(input);
         filename = gjs_argv[1];
         program_name = gjs_argv[1];
     }
@@ -315,11 +341,10 @@ main(int argc, char **argv)
     if (coverage_prefixes)
         gjs_coverage_enable();
 
-    js_context = (GjsContext*) g_object_new(GJS_TYPE_CONTEXT,
-                                            "search-path", include_path,
-                                            "program-name", program_name,
-                                            "profiler-enabled", enable_profiler,
-                                            NULL);
+    js_context = GJS_CONTEXT(g_object_new(
+        GJS_TYPE_CONTEXT, "search-path", include_path, "program-name",
+        program_name, "program-path", program_path.get(), "profiler-enabled",
+        enable_profiler, "exec-as-module", exec_as_module, nullptr));
 
     env_coverage_output_path = g_getenv("GJS_COVERAGE_OUTPUT");
     if (env_coverage_output_path != NULL) {
