@@ -1,9 +1,12 @@
 #!/bin/bash
+# SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+# SPDX-FileCopyrightText: 2018 Claudio André <claudioandre.br@gmail.com>
+
 DIR="$( cd "$( dirname "${0}" )" && pwd )"
 source "${DIR}"/common.sh
 
 # Run the examples
-$gjs examples/gio-cat.js Makefile
+$gjs examples/gio-cat.js meson.build
 report "run the gio-cat.js example"
 
 if [[ -n "${ENABLE_GTK}" ]]; then
