@@ -23,8 +23,9 @@
 #include <js/RootingAPI.h>
 #include <js/TypeDecls.h>
 #include <js/Value.h>     // for NullValue
+#include <js/friend/DumpFunctions.h>
 #include <jsapi.h>        // for JS_DefinePropertyById, JS_DefineF...
-#include <jsfriendapi.h>  // for DumpHeap, IgnoreNurseryObjects
+#include <jsfriendapi.h>  // for GetFunctionNativeReserved, NewFunctionByIdW...
 
 #include "gi/object.h"
 #include "gjs/atoms.h"
@@ -346,7 +347,7 @@ gjs_js_define_system_stuff(JSContext              *context,
                                  GJS_MODULE_PROP_FLAGS | JSPROP_READONLY) &&
            JS_DefinePropertyById(context, module, gjs->atoms().program_args(),
                                  program_args_getter, nullptr,
-                                 GJS_MODULE_PROP_FLAGS | JSPROP_GETTER) &&
+                                 GJS_MODULE_PROP_FLAGS) &&
            JS_DefinePropertyById(context, module, gjs->atoms().version(),
                                  GJS_VERSION,
                                  GJS_MODULE_PROP_FLAGS | JSPROP_READONLY);
